<?php

namespace App\Tools;

use App\Key\K;
use EasySwoole\Component\TableManager;
use SQLite3;
use Swlib\SaberGM;
use Swoole\Coroutine;

class StreetViewDownloader
{
    private $tileUrl = 'https://streetviewpixels-pa.googleapis.com/v1/tile?cb_client=apiv3&panoid={id}&output=tile&x={x}&y={y}&zoom={z}&nbt=1&fover=2';
    /**
     * @var SQLite3
     */
    private $sqlite;

    private $sqliteConfig = [
        'ok' => false,
        'name' => '',
        'sum' => 0,
        'dir' => '',
        'limit' => 4000,
    ];
    /**
     * @var SQLite3Stmt;
     */
    private $insertSt;
    private $wait = [
        'timeout' => 5 * 60,
        'prev_time' => 0
    ];
    private $workerId;
    private $speed = [
        'sum' => 0,
        'minute' => '',
        'ids' => '',
    ];
    private $thread = [
        'sum' => 5,
        'index' => 0
    ];
    private $error = [
        'sum' => 0,
        'list'=>[],
        'limit' => 4000
    ];
    private $zoomList = [
        '0' => ['x' => 0, 'y' => 0, 'need' => true],
        '1' => ['x' => 1, 'y' => 0, 'need' => true],
        '2' => ['x' => 3, 'y' => 1, 'need' => true],
        '3' => ['x' => 6, 'y' => 3, 'need' => true],
        '4' => ['x' => 12, 'y' => 6, 'need' => true],
        '5' => ['x' => 25, 'y' => 12, 'need' => true],
    ];

    public function __construct()
    {
        $this->sqliteConfig['dir'] = dirname(EASYSWOOLE_ROOT . '/db');
        if (!is_dir($this->sqliteConfig['dir'])) mkdir($this->sqliteConfig['dir']);
    }

    public function initSqlite()
    {
        $dbDir = $this->sqliteConfig['dir'];
        $this->sqliteConfig['name'] = date('Y-m-d-H-i-s') . '=' . strtolower(uniqid()) . '.db';
        $this->sqliteConfig['file_dir'] = $dbDir . '/' . $this->sqliteConfig['name'];
        $this->sqlite = new SQLite3($dbDir . '/' . $this->sqliteConfig['name']);
        $this->sqlite->query('CREATE TABLE "tile" (
            "id"	TEXT UNIQUE,
            "data"	BLOB,
            PRIMARY KEY("id")
        )');
        if ($this->sqlite->lastErrorCode()) {
            System::log('create table error', $this->sqlite->lastErrorMsg());
            $this->sqliteConfig['ok'] = false;
            return;
        }
        $this->sqliteConfig['ok'] = true;
        $this->sqlite->query('PRAGMA temp_store = 2');
        $this->insertSt = $this->sqlite->prepare('insert into tile (id,data) values (?,?)');
    }

    public function workerStart($workerId)
    {
        $this->workerId = $workerId;
        if ($workerId === 0) {
            $this->initSqlite();
            $this->mainWorker();
        } else {
            $this->subWorker();
        }
    }

    public function mainWorker()
    {
        while (true) {
            $size = TableManager::getInstance()->get(K::TABLE_STREET_CURL_URL_CACHE)->count();
            if (!$size) {
                $this->noticeError();
                $this->getWork();
            } else {
                $this->saveDbData();
            }
        }
    }

    public function saveDbData()
    {
        Coroutine::sleep(5);
        if (!$this->sqliteConfig['ok']) {
            System::log('sqlite init is no ok');
            return;
        }
        $this->sqlite->query('begin');
        $del = [];
        $write = [
            'done' => 0,
            'new' => 0
        ];
        $dataTable = TableManager::getInstance()->get(K::TABLE_STREET_DATA_CACHE);
        foreach ($dataTable as $k => $v) {
            $del[] = $k;
            $sql = 'select id from tile where id=\'' . $k . '\'';
            $has = $this->sqlite->query($sql);
            if ($this->sqlite->lastErrorCode()) {
                echo $this->sqlite->lastErrorMsg(), PHP_EOL;
                break;
            }
            $has = $has->fetchArray(SQLITE3_ASSOC);
            if ($has['id']) {
                $write['done']++;
                continue;
            }
            $this->insertSt->bindValue('1', $k, SQLITE3_TEXT);
            $this->insertSt->bindValue('2', base64_decode($v['data']), SQLITE3_BLOB);
            $this->insertSt->execute();
            if ($this->sqlite->lastErrorCode()) {
                echo $this->sqlite->lastErrorMsg(), PHP_EOL;
                break;
            }
            $write['new']++;
            $this->sqliteConfig['sum']++;
        }
        foreach ($del as $v) {
            $dataTable->del($v);
        }
        $this->sqlite->query('commit');
        echo 'has new sum ,  ', $write['new'], '   !!done  two ', $write['done'], PHP_EOL;
//            echo 'file size ', filesize($this->sqliteConfig['file_dir']), PHP_EOL;
//            if (filesize($this->sqliteConfig['file_dir'])>$this->sqliteConfig['limit']){
        if ($this->sqliteConfig['sum'] > $this->sqliteConfig['limit']) {
            $this->sqliteConfig['sum'] = 0;
            $this->insertSt->close();
            $this->sqlite->close();
            System::log('change db');
            Coroutine::sleep(1.5);
            if (rename($this->sqliteConfig['file_dir'], __DIR__ . '/db/ok-' . $this->sqliteConfig['name'])) {
                $this->initSqlite();
            } else {
                echo 'rename error from ', $this->sqliteConfig['file_dir'], '  to   ', __DIR__ . '/db/ok-' . $this->sqliteConfig['name'], PHP_EOL;
                $this->sqliteConfig['ok'] = false;
            }
        }
    }

    public function getWork()
    {

        if ($this->speed['minute'] != date('i')) {
            if (!$this->noticeDownloadDbs()) {
                Coroutine::sleep(5);
                System::log('notice db data error');
                return;
            }
            $this->speed['minute'] = date('i');
        }
        if (disk_free_space('/') < 1024 * 1024 * 1024 * 5) {
            Coroutine::sleep(60);
            System::log('too disk space is low');
            return;
        }

        //当前已经下载完成，那么需要进行将下载的数据进行提交
        if (!$this->noticeDownloadIds()) {
            Coroutine::sleep(4);
            return;
        }
        if ($this->speed['time']) {
            echo 'speed,', $this->speed['sum'] / (microtime(true) - $this->speed['time']);
        }
        $this->speed['sum'] = 0;
        $this->speed['time'] = microtime(true);
        $this->createTableData();
        if (TableManager::getInstance()->get(K::TABLE_STREET_CURL_URL_CACHE)->count() === 0) {
            System::log('get no curl list sleep 30s');
            Coroutine::sleep(30);
        }
    }

    public function createTableData()
    {
        try {
            $data = SaberGM::get(K::MANAGE_SERVER_ADDR_9502 . '/street/work/index', ['timeout' => 5]);
            $all = $data->getBody()->getContents();
            $all = json_decode($all, true);
            $table = TableManager::getInstance()->get(K::TABLE_STREET_CURL_URL_CACHE);
            //将当前的ID要缓存起来，这样在下载完成 之后， 要通知主服务器
            $this->speed['ids'] = $all['data']['ids'];
            foreach ($all['data']['ids'] as $v) {
                $tiles = $this->createTiles($v['id']);
                $this->speed['sum'] += count($tiles);
                foreach ($tiles as $vv) {
                    $table->set($vv['tile_id'], ['data' => json_encode($vv)]);
                }
            }
            echo 'current  loop sum,', $this->speed['sum'], PHP_EOL;
        } catch (\Throwable $throwable) {
            System::log('get curl url failed ', $throwable->getMessage());
        }
    }

    public function createTiles($id)
    {
        $r = [];
        foreach ($this->zoomList as $z => $v) {
            if ($v['need']) {
                for ($i = 0; $i <= $v['x']; $i++) {
                    for ($j = 0; $j <= $v['y']; $j++) {
                        $r[] = [
                            'id' => $id,
                            'tile_id' => implode('-', [$id, $z, $i, $j]),
                            'url' => str_replace(['{id}', '{z}', '{x}', '{y}'], [$id, $z, $i, $j], $this->tileUrl),
                        ];
                    }
                }
            }
        }
        return $r;
    }


    public function curl($data)
    {
        try {
            $port = 7890;
//            echo $data['url'],PHP_EOL;
            $dataTable = TableManager::getInstance()->get(K::TABLE_STREET_DATA_CACHE);
            $result = SaberGM::get($data['url'], ['timeout' => 30]);
            $result = $result->getBody()->getContents();
            if ($dataTable->get($data['tile_id'])) {
                echo 'has tile_data', PHP_EOL;
            } else {
                $dataTable->set($data['tile_id'], ['data' => base64_encode($result)]);
            }
        } catch (\Throwable $throwable) {
            $code = $throwable->getCode();
            echo $throwable->getMessage(), PHP_EOL;
            $table = TableManager::getInstance()->get(K::TABLE_STREET_CURL_URL_CACHE);
            switch ($code) {
                case -1:
                case -2:
                case -4:
                case -3:
                    $table->set($data['tile_id'], ['data' => json_encode($data)]);
                    break;
                default:
                    $this->error['list'][]=['id'=>$data['tile_id'],'data'=>$data];
                    echo 'worker_id ', $this->workerId, ' === ', $throwable->getMessage(), PHP_EOL;
                    $this->error['sum']++;
            }
        }
    }


    public function subWorker()
    {

        for ($i = 0; $i < $this->thread['sum']; $i++) {
            go([$this, 'download']);
        }
    }

    public function download()
    {
        $table = TableManager::getInstance()->get(K::TABLE_STREET_CURL_URL_CACHE);
        $count = $table->count();
        if ($count && $count % 100 === 0) {
            echo 'count ', $count, ' error', $this->error['sum'], PHP_EOL;
        }
        if (!$count) {
            Coroutine::sleep(2);
            $this->download();
        } else {
            if ($this->error['sum'] > $this->error['limit']) {
                echo 'too many errors', PHP_EOL;
                return;
            }
            $this->wait['prev_time'] = 0;
            if (TableManager::getInstance()->get(K::TABLE_STREET_DATA_CACHE)->count() > K::TABLE_SIZE_STREET_DATA * .9) {
                System::log('size too big wait');
                Coroutine::sleep(2);
            } else {
                $data = [];
                $i = 0;
                foreach ($table as $k => $v) {
                    $i++;
                    $data[] = ['value' => json_decode($v['data'], true), 'k' => $k];
                    break;
                }
                shuffle($data);
                $tmp = current($data);
                unset($data);
                $table->del($tmp['k']);
                if ($tmp['value']) {
                    $this->curl($tmp['value']);
                }
            }
            $this->download();
        }
    }

    private function noticeDownloadIds()
    {
        $result = false;
        try {
            SaberGM::post(K::MANAGE_SERVER_ADDR_9502 . '/street/work/confirm', ['ids' => $this->speed['ids']], ['timeout' => 5]);
            $result = true;
        } catch (\Throwable $throwable) {
            System::log('upload download ids error', $throwable->getMessage());
        }
        return $result;
    }

    private function noticeError(){
        if (!$this->error['list']) return;
        try {
            SaberGM::post(K::MANAGE_SERVER_ADDR_9502.'/street/work/error',$this->error['list'],['timeout'=>5]);
            $this->error['list']=[];
        }catch (\Throwable $throwable){
            System::log('upload error list error '.$throwable->getMessage());
        }
    }

    private function noticeDownloadDbs()
    {
        $result = false;
        try {
            $dbs = scandir($this->sqliteConfig['dir']);
            $dbs = array_filter($dbs, function ($v) {
                return strtolower(pathinfo($v, PATHINFO_EXTENSION)) === 'db';
            });
            foreach ($dbs as $k => $v) {
                if ($v === $this->sqliteConfig['name']) unset($dbs[$k]);
            }
            $dbs = array_values($dbs);
            SaberGM::post(K::MANAGE_SERVER_ADDR_9502 . '/street/work/db', ['dbs' => $dbs], ['timeout' => 5]);
            $result = true;
        } catch (\Throwable $throwable) {
            System::log('upload download ids error', $throwable->getMessage());
        }
        return $result;
    }
}
